function read_envstruct,ext=ext
if n_elements(ext) eq 0 then begin
   ext=''
endif
file='envstruct'+ext+'.dat'
openr,1,file
nr=0
readf,1,nr
dat=dblarr(3,nr)
readf,1,dat
close,1
r=transpose(dat(0,*))
rho=transpose(dat(1,*))
temp=transpose(dat(2,*))
return,{r:r,rho:rho,temp:temp}
end


function read_convhist
openr,1,'convhist.info'
nn=0
readf,1,nn
close,1
openr,1,'convhist.dat'
nr=1
readf,1,nr
dat=dblarr(9,nr,nn)
readf,1,dat
close,1
if nn gt 1 then idx=[1,2,0] else idx=[1,0]
temp=transpose(dat(0,*,*),idx)
jjme=transpose(dat(1,*,*),idx)
hhme=transpose(dat(2,*,*),idx)
jj=transpose(dat(3,*,*),idx)
hh=transpose(dat(4,*,*),idx)
kapt=transpose(dat(5,*,*),idx)
kapj=transpose(dat(6,*,*),idx)
kaph=transpose(dat(7,*,*),idx)
fj=transpose(dat(8,*,*),idx)
openr,1,'envstruct.inp'
nnr=1
readf,1,nnr
dat=dblarr(3,nnr)
readf,1,dat
r=transpose(dat(0,*))
close,1
return,{r:r,temp:temp,jjme:jjme,hhme:hhme,jj:jj,hh:hh,kapt:kapt,$
        kapj:kapj,kaph:kaph,fj:fj}
end



function read_spectrum,ext=ext,file=file
fr_units=0
;
; Read possible dust information
;
z=findfile('dustopac.inp',count=count)
dust=count
if dust gt 0 then begin
    print,"Found dustopac.inp, so assuming dust spectrum"
    fr_units = -1
endif

;
; Read the total spectrum
;
if not keyword_set(file) then begin
   if n_elements(ext) eq 0 then begin
      filename='spectrum.dat'
   endif else begin
      filename='spectrum_'+strcompress(string(ext),/remove_all)+'.dat'
   endelse
endif else begin
   filename=file
endelse
openr,1,filename
nrfr=1
readf,1,nrfr
dum=dblarr(2,nrfr)
freq=dblarr(nrfr)
spectrum=dblarr(nrfr)
readf,1,dum
freq(0:nrfr-1) = dum(0,0:nrfr-1)
spectrum(0:nrfr-1) = dum(1,0:nrfr-1)
close,1
;
a={spectrum:spectrum,nfr:nrfr,freq:freq,fr_units:fr_units,obs:0}
return,a
end



pro plot_spectrum,a,ev=ev,kev=kev,hz=hz,micron=micron,$
        lnu=lnu,fnu=fnu,nulnu=nulnu,nufnu=nufnu,rpc=rpc,$
        dpc=dpc,xlg=xlg,oplot=oplot,jy=jy,lsun=lsun,$
        itheta=itheta,ldat=ldat,ylin=ylin,lum=lum,$
        thick=thick,xthick=xthick,ythick=ythick,$
        charthick=charthick,charsize=charsize,_extra=_extra
if not keyword_set(itheta) then itheta=0
fr_units = a.fr_units
if keyword_set(micron) then begin
    fr_units = -1
endif
if keyword_set(hz) then begin
    fr_units = 0
endif
if keyword_set(ev) then begin
    fr_units = 1
endif
if keyword_set(kev) then begin
    fr_units = 2
endif
case fr_units of
    -1: begin
        xcoord = 2.9979d14 / a.freq(*)
        xtitle = '!4k!X [!4l!Xm]'
    end
    0: begin
        xcoord = a.freq(*)
        xtitle = '!4m!X [Hz]'
    end
    1: begin
        xcoord = 4.13568842841d-15 * a.freq(*)
        xtitle = '!4m!X [eV]'
    end
    2: begin
        xcoord = 4.13568842841d-18 * a.freq(*)
        xtitle = '!4m!X [KeV]'
    end
endcase

if keyword_set(dpc) then rpc=dpc
;
; Plot nuFnu or Fnu (same with Lnu)? And what about Fnu vs Lnu?
;
sed=1
ylum=0
if keyword_set(jy) then begin
   sed=0
endif
if keyword_set(fnu) then begin
   ylum=0
   sed=0
endif
if keyword_set(lnu) then begin
   ylum=1
   sed=0
endif
if keyword_set(nufnu) then begin
   ylum=0
   sed=1
endif
if keyword_set(nulnu) then begin
   ylum=1
   sed=1
endif
if keyword_set(lum) then ylum=1 
if keyword_set(jy) then begin
   ylum=0
   ;sed=0
endif
if keyword_set(lsun) then begin
   ylum=1
   sed=1
endif
;
; If the distance is not given, then for an observed spectrum
; we can only plot the flux, while for a computed spectrum we
; can only plot the luminosity
;
if a.obs eq 0 then begin
   if not keyword_set(rpc) and keyword_set(jy) then begin
      print,"Cannot use Jy units of theoretical spectrum if dpc not known."
      return
   endif
   if not keyword_set(rpc) and ylum eq 0 then begin
      ylum=1
      print,"Do not know distance to source. Plotting Luminosity instead."
   endif
endif else begin
   if not keyword_set(rpc) and ylum eq 1 then begin
      ylum=0
      print,"Do not know distance to source. Plotting Flux instead."
   endif
endelse
;
; Which plot to make? Lum or flux?
;
if ylum eq 0 then begin
   ;
   ; Plot spectrum as flux at a certain distance
   ;
   if a.obs eq 0 then begin
      rpc=1.d0*rpc
      distfact = 1.d0/ (rpc^2)
   endif else begin
      distfact = 1.d0
   endelse
   if not keyword_set(jy) then begin
      if sed eq 0 then begin
         lumfact=1.d0
         ytitle='F!I!4m!X!N  [erg cm!E-2!N Hz!E-1!N s!E-1!N]'
      endif else begin
         lumfact=a.freq(*)
         ytitle='!4m!XF!I!4m!X!N [erg cm!E-2!N s!E-1!N]'
      endelse
   endif else begin
      if sed eq 0 then begin
         lumfact=1d+23
         ytitle='F!I!4m!X!N [Jy]'
      endif else begin
         lumfact=1d+23*a.freq(*)
         ytitle='!4m!XF!I!4m!X!N [JyHz]'
      endelse
   endelse
endif else begin
   ;
   ; Plot spectrum as luminosity
   ;
   if a.obs eq 0 then begin
      distfact = 1.1965280793d38 ; = (1 parsec)^2 = 1.19d38 cm^2
   endif else begin
      rpc=1.d0*rpc
      distfact = rpc^2 * 1.1965280793d38
   endelse
   if sed eq 0 then begin
      lumfact=1.d0
      ytitle='L!I!4m!X!N  [erg Hz!E-1!N s!E-1!N]'
   endif else begin
      if not keyword_set(lsun) then begin
         lumfact=a.freq(*)
         ytitle='!4m!XL!I!4m!X!N [erg s!E-1!N]'
      endif else begin
         lumfact=a.freq(*)*2.5956986d-34
         ytitle='!4m!XL!I!4m!X!N [Lsun]'
      endelse
   endelse
endelse
if not keyword_set(xlg) then begin
    xlog=1
endif else begin
    xcoord=alog10(xcoord)
    xtitle = 'log '+xtitle
    xlog=0
endelse
if not keyword_set(ylin) then begin
    ylog=1
endif else begin
    ylog=0
endelse
if not keyword_set(oplot) then begin
    plot,xcoord,distfact*lumfact*a.spectrum(*,itheta),$
         xtitle=xtitle,ytitle=ytitle,$
         xlog=xlog,ylog=ylog,thick=thick,xthick=xthick,ythick=ythick,$
        charthick=charthick,charsize=charsize,_extra=_extra
endif else begin
    oplot,xcoord,distfact*lumfact*a.spectrum(*,itheta),$
         thick=thick,_extra=_extra
endelse
end
